import unittest

from ..compose_generator import ComposeGenerator


class TestComposeServiceEnvironmentFile(unittest.TestCase):
    def test_service_with_single_environment_file(self):
        compose_file = ComposeGenerator.get_compose_with_environment_file_single()
        environment_from_file = compose_file.services["frontend"].env_file.readFile()

        self.assertEqual(compose_file.services["frontend"].env_file[0], "./sample/test.env")
        self.assertEqual(environment_from_file["RACK_ENV"], "development")
        self.assertEqual(environment_from_file["VAR"], '"quoted"')

    def test_service_with_list_environment_file(self):
        compose_file = ComposeGenerator.get_compose_with_environment_file_list()
        environment_from_file = compose_file.services["frontend"].env_file.readFile()

        self.assertEqual(compose_file.services["frontend"].env_file, ['./sample/common.env', './sample/apps/web.env', './sample/opt/runtime_opts.env'])
        self.assertEqual(environment_from_file["RACK_ENV"], "development")
        self.assertEqual(environment_from_file["VAR"], '"quoted"')
        self.assertEqual(environment_from_file["BAR"], "faz")
        self.assertEqual(environment_from_file["FOO"], '"bar"')
        self.assertEqual(environment_from_file["BAZ"], "snafu")
        self.assertEqual(environment_from_file["STAGE"], "dev")
        self.assertEqual(environment_from_file["SHOW"], "false")

    def test_service_with_resolved_environment_variables(self):
        compose_file = ComposeGenerator.get_compose_with_environment_file_list()
        environment_from_file = compose_file.services["frontend"].resolve_environment_hierarchy()

        self.assertEqual(environment_from_file["RACK_ENV"], "canary")
        self.assertEqual(environment_from_file["VAR"], '"quoted"')
        self.assertEqual(environment_from_file["BAR"], "faz")
        self.assertEqual(environment_from_file["FOO"], '"bar"')
        self.assertEqual(environment_from_file["BAZ"], "snafu")
        self.assertEqual(environment_from_file["STAGE"], "dev")
        self.assertEqual(environment_from_file["SHOW"], "true")
        self.assertIsNone(environment_from_file["USER_INPUT"])



from pycomposefile.compose_element import ComposeElement, ComposeStringOrListElement


class HealthCheck(ComposeElement):
    element_keys = {
        "test": (ComposeStringOrListElement, "https://github.com/compose-spec/compose-spec/blob/master/spec.md#healthcheck"),
        "interval": (str, "https://github.com/compose-spec/compose-spec/blob/master/spec.md#healthcheck"),
        "timeout": (str, "https://github.com/compose-spec/compose-spec/blob/master/spec.md#healthcheck"),
        "retries": (int, "https://github.com/compose-spec/compose-spec/blob/master/spec.md#healthcheck"),
        "start_period": (str, "https://github.com/compose-spec/compose-spec/blob/master/spec.md#healthcheck"),
    }



from .compose_element import ComposeElement, ComposeByteValue
from .compose_list_or_map import ComposeListOrMapElement
from .compose_string_or_list import ComposeStringOrListElement
from .compose_datatype_transformer import ComposeDataTypeTransformer, EmptyOrUnsetException



from pycomposefile.compose_element import (ComposeElement,
                                           ComposeByteValue,
                                           ComposeStringOrListElement)


class Expose(ComposeStringOrListElement):
    transform = int


class Dependency(str):
    def __new__(cls, config, key=None, compose_path=None, ) -> None:
        condition = None
        if isinstance(config, Dependency):
            return config
        if isinstance(config, tuple):
            name, detail = config
            condition = detail["condition"]
        else:
            name = config
        ob = super(Dependency, cls).__new__(cls, name)
        ob.__setattr__('condition', condition)
        return ob


class DependsOn(ComposeStringOrListElement):
    def __init__(self, config, key=None, compose_path=None):
        self.transform = Dependency
        if isinstance(config, dict):
            config_to_list = []
            for key in config.keys():
                config_to_list.append((key, config[key]))
            config = config_to_list
        super().__init__(config, key, compose_path)


class StorageOpt(ComposeElement):
    element_keys = {
        "size": (ComposeByteValue, "https://github.com/compose-spec/compose-spec/blob/master/spec.md#storage_opt")
    }


class Nofile(ComposeElement):
    element_keys = {
        "hard": (int, ""),
        "soft": (int, ""),
    }


class Ulimits(ComposeElement):
    element_keys = {
        "nproc": (int, ""),
        "nofile": (Nofile.from_parsed_yaml, "")
    }

    @classmethod
    def from_parsed_yaml(cls, config, name, compose_path):
        if config is None:
            return None
        if isinstance(config, str) or isinstance(config, int):
            new_dict = {}
            new_dict["nproc"] = config
            config = new_dict
        compose_path = f"{compose_path}/{name}"
        return cls(config, compose_path)



from pycomposefile.compose_element import ComposeElement, ComposeByteValue, ComposeStringOrListElement, ComposeListOrMapElement
from pycomposefile.service.service_environment import Environment
from pycomposefile.service.service_configs import Secrets


class Build(ComposeElement):
    element_keys = {
        "context": (str, "https://github.com/compose-spec/compose-spec/blob/master/build.md#context-required"),
        "dockerfile": (str, "https://github.com/compose-spec/compose-spec/blob/master/build.md#dockerfile"),
        "args": (Environment, "https://github.com/compose-spec/compose-spec/blob/master/build.md#args"),
        "ssh": (ComposeStringOrListElement, "https://github.com/compose-spec/compose-spec/blob/master/build.md#ssh"),
        "cache_from": (ComposeStringOrListElement, "https://github.com/compose-spec/compose-spec/blob/master/build.md#cache_from"),
        "cache_to": (ComposeStringOrListElement, "https://github.com/compose-spec/compose-spec/blob/master/build.md#cache_to"),
        "extra_hosts": (ComposeStringOrListElement, "https://github.com/compose-spec/compose-spec/blob/master/build.md#extra_hosts"),
        "isolation": (str, "https://github.com/compose-spec/compose-spec/blob/master/build.md#isolation"),
        "labels": (ComposeListOrMapElement, "https://github.com/compose-spec/compose-spec/blob/master/build.md#labels"),
        "no_cache": (bool, "https://github.com/compose-spec/compose-spec/blob/master/build.md#no_cache"),
        "pull": (bool, "https://github.com/compose-spec/compose-spec/blob/master/build.md#pull"),
        "shm_size": (ComposeByteValue, "https://github.com/compose-spec/compose-spec/blob/master/build.md#shm_size"),
        "target": (str, "https://github.com/compose-spec/compose-spec/blob/master/build.md#target"),
        "secrets": (Secrets, "https://github.com/compose-spec/compose-spec/blob/master/build.md#secrets"),
        "tags": (ComposeListOrMapElement, "https://github.com/compose-spec/compose-spec/blob/master/build.md#tags")
    }



import unittest

from ..compose_generator import ComposeGenerator


class TestComposeServiceEnvironmentFile(unittest.TestCase):
    def test_service_with_single_environment_file(self):
        compose_file = ComposeGenerator.get_compose_with_network()

        self.assertEqual(compose_file.services["grafana"].network_mode, "bridge")
